@@ -141,6 +141,18 @@ class ADUserService:
         #ad_users = [ADUser(**x) for x in ad_users_dict]
         return ad_users
 
+    def get_all_sam_account_names(self, search_filter: str = '(objectClass=user)') -> set[str]:
+        """Retrieve all sAMAccountName values from Active Directory.
+
+        Returns:
+            set[str]: A set of all sAMAccountName values.
+        """
+        attributes = ['sAMAccountName']
+        ad_users_dict = self.ad_connection.search(search_filter, attributes)
+        sam_account_names = {str(user.get('sAMAccountName')) for user
+                             in ad_users_dict if 'sAMAccountName' in user}
+        return sam_account_names
+
     def add_member(self, user: ADUser, group_dn: str) -> dict[str, Any]:
         """Add a user to an Active Directory group.
 
@@ -178,7 +190,7 @@ class ADUserService:
 
         current_dn = user.distinguishedName
         try:
-            response = self.ad_connection.move_entry(current_dn, target_ou_dn)
+            response = self.ad_connection.move_entry(str(current_dn), target_ou_dn)
         except LDAPException as e:
             self.logger.error(
                 "Exception occurred while moving user %s to %s: %s",
@@ -198,9 +210,9 @@ class ADUserService:
             )
             return result
 
-        new_dn = f"{current_dn.split(',', 1)[0]},{target_ou_dn}"
-        user.distinguishedName = new_dn
-        user.ou = target_ou_dn
+        new_dn = f"CN={user.sAMAccountName},{target_ou_dn}"
+        setattr(user, 'distinguishedName', new_dn)
+        setattr(user, 'ou', target_ou_dn)
         result['dn'] = new_dn
         self.logger.info(
             "Moved %s to new OU %s",
@@ -230,6 +242,8 @@ class ADUserService:
         change_affects = {k: f"{getattr(user, k)} -> {v}" for k, v in changes}
         try:
             response = self.ad_connection.modify(user.distinguishedName, changes)
+            for k, v in changes:
+                setattr(user, k, v)
         except LDAPException as e:
             self.logger.error(
                 "Exception occurred while modifying user %s: changes: %s error msg: %s",
@@ -278,6 +292,31 @@ class ADUserService:
     def _user_dn(self, cn: str, ou_dn: str) -> str:
         return f"CN={cn},{ou_dn}"
 
+    def set_password(self, user: ADUser, new_password: str, must_change_at_next_logon: bool = True
+                     ) -> dict[str, Any]:
+        """Set a new password for the specified user.
+        
+        Args:
+            user (ADUser): The user for whom to set the password.
+            new_password (str): The new password to set.
+            must_change_at_next_logon (bool): Whether the user must change the password at next
+                logon. Defaults to True.
+        """
+        try:
+            response = self.ad_connection.set_password(user.distinguishedName, new_password,
+                                                       must_change_at_next_logon)
+            if not response.get('success'):
+                self.logger.warning("Failed to set password for user %s: %s", user.sAMAccountName,
+                                    response.get('result'))
+                return {'success': False, 'result': response.get('result')}
+        except LDAPException as e:
+            self.logger.error("Exception occurred while setting password for user %s: %s",
+                              user.sAMAccountName, str(e))
+            return {'success': False, 'result': str(e)}
+
+        self.logger.info("Successfully set password for user %s", user.sAMAccountName)
+        return {'success': True, 'result': response.get('result')}
+
     def create_user(
         self,
         cn: str,
@@ -328,7 +367,10 @@ class ADUserService:
                 # You may choose to return failure or proceed; here we surface partial failure:
                 return {
                     'success': False,
-                    'result': {'create': response.get('result'), 'password': pw_resp.get('result')},
+                    'result': {
+                        'create': response.get('result'),
+                        'password': pw_resp.get('result')
+                    },
                     'dn': dn,
                 }
 
